# Copyright 2021 Dominik George <nik@naturalnet.de>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import toml

DEFAULT_CONFIG_FILE = "/etc/nss_pam_oidc.toml"
DEFAULT_CONFIG = {
    "pam": {
        "flow": "password",
    },
    "nss": {},
}


def load_config(config_file: str) -> dict[str, dict[str, str]]:
    """Load full configuration, amended wit hdefault config."""
    config = {}

    # First, copy default configuration
    for name, conf in DEFAULT_CONFIG.items():
        config[name] = conf.copy()

    # Second, load configuration from file
    if os.path.exists(config_file):
        config_toml = toml.load(config_file)
        for name, conf in config.items():
            conf.update(config_toml.get(name, {}))

    return config


def get_config(section: str, **kwargs) -> dict[str, str]:
    """Get configuration for a section, layered from defaults, config file, and args."""
    config_file = kwargs.pop("config", DEFAULT_CONFIG_FILE)
    config = load_config(config_file)

    # Lastly, override with passed arguments
    config[section].update(kwargs)

    return config[section]


def filter_config(config: dict[str, str], keys: list[str]) -> dict[str, str]:
    """Return a copy of the config dictionary with only the requested keys."""
    return {k: v for k, v in config.items() if k in keys}
