# Copyright 2021 Dominik George <nik@naturalnet.de>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oauthlib.oauth2 import InvalidGrantError, LegacyApplicationClient
from requests.exceptions import RequestException
from requests_oauthlib import OAuth2Session

from .config import filter_config, get_config


def _split_argv(argv: list[str]) -> dict[str, str]:
    args = {}
    for arg in argv:
        if "=" in arg:
            name, val = arg.split("=", 1)
        else:
            name, val = arg, True
        args[name] = val
    return args


def _do_legacy_auth(username: str, password: str, config: dict[str, str]):
    client_config = filter_config(config, ["client_id"])
    session_config = filter_config(config, ["client_id"])
    fetch_config = filter_config(config, ["client_id", "client_secret", "token_url"])

    client = LegacyApplicationClient(**client_config)
    session = OAuth2Session(client=client, **session_config)

    token = session.fetch_token(username=username, password=password, **fetch_config)
    return token


def pam_sm_authenticate(pamh, flags, argv):
    args = _split_argv(argv)
    config = get_config("pam", args)

    if config.pop("flow") == "password":
        try:
            user = pamh.get_user(None)
            password = pamh.authtok
        except pamh.exception as e:
            return e.pam_result
        if user is None or password is None:
            return pamh.PAM_CRED_INSUFFICIENT

        try:
            token = _do_legacy_auth(username, password)
        except InvalidGrantError:
            return pamh.PAM_AUTH_ERROR
        except RequestException:
            return pamh.PAM_AUTHINFO_UNAVAIL
        except:
            return pamh.PAM_SERVICE_ERR

    if "access_token" in token:
        return pamh.PAM_SUCCESS

    return pamh.PAM_AUTH_ERR


def pam_sm_setcred(pamh, flags, argv):
    return pamh.PAM_SUCCESS


def pam_sm_acct_mgmt(pamh, flags, argv):
    return pamh.PAM_SUCCESS


def pam_sm_open_session(pamh, flags, argv):
    return pamh.PAM_SUCCESS


def pam_sm_close_session(pamh, flags, argv):
    return pamh.PAM_SUCCESS


def pam_sm_chauthtok(pamh, flags, argv):
    return pamh.PAM_SUCCESS
